@echo off

rem ---------------------------------
rem ---- run everything in the bin
bin\Debug\addmoddel.exe
bin\Debug\convert-test.exe
bin\Debug\exifcomment.exe
bin\Debug\exifdata-test.exe
bin\Debug\exifprint.exe
bin\Debug\exiv2.exe
bin\Debug\iotest.exe
bin\Debug\iptceasy.exe
bin\Debug\iptcprint.exe
bin\Debug\iptctest.exe
bin\Debug\largeiptc-test.exe
bin\Debug\metacopy.exe
bin\Debug\mmap-test.exe
bin\Debug\prevtest.exe
bin\Debug\stringto-test.exe
bin\Debug\taglist.exe
bin\Debug\tiff-test.exe
bin\Debug\utiltest.exe
bin\Debug\write-test.exe
bin\Debug\write2-test.exe
bin\Debug\xmpparse.exe
bin\Debug\xmpparser-test.exe
bin\Debug\xmpsample.exe
bin\DebugDLL\addmoddel.exe
bin\DebugDLL\convert-test.exe
bin\DebugDLL\exifcomment.exe
bin\DebugDLL\exifdata-test.exe
bin\DebugDLL\exifprint.exe
bin\DebugDLL\exiv2.exe
bin\DebugDLL\iotest.exe
bin\DebugDLL\iptceasy.exe
bin\DebugDLL\iptcprint.exe
bin\DebugDLL\iptctest.exe
bin\DebugDLL\largeiptc-test.exe
bin\DebugDLL\metacopy.exe
bin\DebugDLL\mmap-test.exe
bin\DebugDLL\prevtest.exe
bin\DebugDLL\stringto-test.exe
bin\DebugDLL\taglist.exe
bin\DebugDLL\tiff-test.exe
bin\DebugDLL\utiltest.exe
bin\DebugDLL\write-test.exe
bin\DebugDLL\write2-test.exe
bin\DebugDLL\xmpparse.exe
bin\DebugDLL\xmpparser-test.exe
bin\DebugDLL\xmpsample.exe
bin\Release\addmoddel.exe
bin\Release\convert-test.exe
bin\Release\exifcomment.exe
bin\Release\exifdata-test.exe
bin\Release\exifprint.exe
bin\Release\exiv2.exe
bin\Release\iotest.exe
bin\Release\iptceasy.exe
bin\Release\iptcprint.exe
bin\Release\iptctest.exe
bin\Release\largeiptc-test.exe
bin\Release\metacopy.exe
bin\Release\mmap-test.exe
bin\Release\prevtest.exe
bin\Release\stringto-test.exe
bin\Release\taglist.exe
bin\Release\tiff-test.exe
bin\Release\utiltest.exe
bin\Release\write-test.exe
bin\Release\write2-test.exe
bin\Release\xmpparse.exe
bin\Release\xmpparser-test.exe
bin\Release\xmpsample.exe
bin\ReleaseDLL\addmoddel.exe
bin\ReleaseDLL\convert-test.exe
bin\ReleaseDLL\exifcomment.exe
bin\ReleaseDLL\exifdata-test.exe
bin\ReleaseDLL\exifprint.exe
bin\ReleaseDLL\exiv2.exe
bin\ReleaseDLL\iotest.exe
bin\ReleaseDLL\iptceasy.exe
bin\ReleaseDLL\iptcprint.exe
bin\ReleaseDLL\iptctest.exe
bin\ReleaseDLL\largeiptc-test.exe
bin\ReleaseDLL\metacopy.exe
bin\ReleaseDLL\mmap-test.exe
bin\ReleaseDLL\prevtest.exe
bin\ReleaseDLL\stringto-test.exe
bin\ReleaseDLL\taglist.exe
bin\ReleaseDLL\tiff-test.exe
bin\ReleaseDLL\utiltest.exe
bin\ReleaseDLL\write-test.exe
bin\ReleaseDLL\write2-test.exe
bin\ReleaseDLL\xmpparse.exe
bin\ReleaseDLL\xmpparser-test.exe
bin\ReleaseDLL\xmpsample.exe

rem ---------------------------------
rem ---- run everything in the build directories
addmoddel\Debug\addmoddel.exe
addmoddel\DebugDLL\addmoddel.exe
addmoddel\Release\addmoddel.exe
addmoddel\ReleaseDLL\addmoddel.exe
convert-test\Debug\convert-test.exe
convert-test\DebugDLL\convert-test.exe
convert-test\Release\convert-test.exe
convert-test\ReleaseDLL\convert-test.exe
exifcomment\Debug\exifcomment.exe
exifcomment\DebugDLL\exifcomment.exe
exifcomment\Release\exifcomment.exe
exifcomment\ReleaseDLL\exifcomment.exe
tiff-test\Debug\tiff-test.exe
tiff-test\DebugDLL\tiff-test.exe
tiff-test\Release\tiff-test.exe
tiff-test\ReleaseDLL\tiff-test.exe
exifdata-test\Debug\exifdata-test.exe
exifdata-test\DebugDLL\exifdata-test.exe
exifdata-test\Release\exifdata-test.exe
exifdata-test\ReleaseDLL\exifdata-test.exe
exifprint\Debug\exifprint.exe
exifprint\DebugDLL\exifprint.exe
exifprint\Release\exifprint.exe
exifprint\ReleaseDLL\exifprint.exe
exiv2\Debug\exiv2.exe
exiv2\DebugDLL\exiv2.exe
exiv2\Release\exiv2.exe
exiv2\ReleaseDLL\exiv2.exe
iotest\Debug\iotest.exe
iotest\DebugDLL\iotest.exe
iotest\Release\iotest.exe
iotest\ReleaseDLL\iotest.exe
iptceasy\Debug\iptceasy.exe
iptceasy\DebugDLL\iptceasy.exe
iptceasy\Release\iptceasy.exe
iptceasy\ReleaseDLL\iptceasy.exe
iptcprint\Debug\iptcprint.exe
iptcprint\DebugDLL\iptcprint.exe
iptcprint\Release\iptcprint.exe
iptcprint\ReleaseDLL\iptcprint.exe
iptctest\Debug\iptctest.exe
iptctest\DebugDLL\iptctest.exe
iptctest\Release\iptctest.exe
iptctest\ReleaseDLL\iptctest.exe
largeiptc-test\Debug\largeiptc-test.exe
largeiptc-test\DebugDLL\largeiptc-test.exe
largeiptc-test\Release\largeiptc-test.exe
largeiptc-test\ReleaseDLL\largeiptc-test.exe
metacopy\Debug\metacopy.exe
metacopy\DebugDLL\metacopy.exe
metacopy\Release\metacopy.exe
mmap-test\ReleaseDLL\mmap-test.exe
mmap-test\Debug\mmap-test.exe
mmap-test\DebugDLL\mmap-test.exe
mmap-test\Release\mmap-test.exe
metacopy\ReleaseDLL\metacopy.exe
prevtest\Debug\prevtest.exe
prevtest\DebugDLL\prevtest.exe
prevtest\Release\prevtest.exe
prevtest\ReleaseDLL\prevtest.exe
stringto-test\Debug\stringto-test.exe
stringto-test\DebugDLL\stringto-test.exe
stringto-test\Release\stringto-test.exe
stringto-test\ReleaseDLL\stringto-test.exe
taglist\Debug\taglist.exe
taglist\DebugDLL\taglist.exe
taglist\Release\taglist.exe
taglist\ReleaseDLL\taglist.exe
tiff-test\Debug\tiff-test.exe
tiff-test\DebugDLL\tiff-test.exe
tiff-test\Release\tiff-test.exe
tiff-test\ReleaseDLL\tiff-test.exe
utiltest\Debug\utiltest.exe
utiltest\DebugDLL\utiltest.exe
utiltest\Release\utiltest.exe
utiltest\ReleaseDLL\utiltest.exe
write-test\Debug\write-test.exe
write-test\DebugDLL\write-test.exe
write-test\Release\write-test.exe
write-test\ReleaseDLL\write-test.exe
write2-test\Debug\write2-test.exe
write2-test\DebugDLL\write2-test.exe
write2-test\Release\write2-test.exe
write2-test\ReleaseDLL\write2-test.exe
xmpparse\Debug\xmpparse.exe
xmpparse\DebugDLL\xmpparse.exe
xmpparse\Release\xmpparse.exe
xmpparse\ReleaseDLL\xmpparse.exe
xmpparser-test\Debug\xmpparser-test.exe
xmpparser-test\DebugDLL\xmpparser-test.exe
xmpparser-test\Release\xmpparser-test.exe
xmpparser-test\ReleaseDLL\xmpparser-test.exe
xmpsample\Debug\xmpsample.exe
xmpsample\DebugDLL\xmpsample.exe
xmpsample\Release\xmpsample.exe
xmpsample\ReleaseDLL\xmpsample.exe

rem ---------------------------------
rem ------- exiv2.exe on a file ----
bin\Debug\exiv2.exe      -pt test.jpg
bin\Debug\exiv2.exe          test.png
bin\Debug\exiv2.exe          test.tiff
bin\DebugDLL\exiv2.exe   -pt test.jpg
bin\DebugDLL\exiv2.exe       test.png
bin\DebugDLL\exiv2.exe       test.tiff
bin\Release\exiv2.exe    -pt test.jpg
bin\Release\exiv2.exe        test.png
bin\Release\exiv2.exe        test.tiff
bin\ReleaseDLL\exiv2.exe -pt test.jpg
bin\ReleaseDLL\exiv2.exe     test.png
bin\ReleaseDLL\exiv2.exe     test.tiff

rem ---------------------------------
rem ------- mmap-test on a file ----
bin\Debug\mmap-test.exe        test.tiff
bin\DebugDLL\mmap-test.exe     test.tiff
bin\Release\mmap-test.exe      test.tiff
bin\ReleaseDLL\mmap-test.exe   test.tiff

rem ---------------------------------
rem Check the DLL dependencies
cd bin\Debug
echo -------Debug---------------
..\..\depends2.exe addmoddel.exe         | sort
..\..\depends2.exe convert-test.exe      | sort
..\..\depends2.exe exifcomment.exe       | sort
..\..\depends2.exe exifdata-test.exe     | sort
..\..\depends2.exe exifprint.exe         | sort
..\..\depends2.exe exiv2.exe             | sort
..\..\depends2.exe exivsimple.dll        | sort
..\..\depends2.exe iotest.exe            | sort
..\..\depends2.exe iptceasy.exe          | sort
..\..\depends2.exe iptcprint.exe         | sort
..\..\depends2.exe iptctest.exe          | sort
..\..\depends2.exe largeiptc-test.exe    | sort
..\..\depends2.exe metacopy.exe          | sort
..\..\depends2.exe mmap-test.exe         | sort
..\..\depends2.exe prevtest.exe          | sort
..\..\depends2.exe stringto-test.exe     | sort
..\..\depends2.exe taglist.exe           | sort
..\..\depends2.exe tiff-test.exe         | sort
..\..\depends2.exe utiltest.exe          | sort
..\..\depends2.exe write-test.exe        | sort
..\..\depends2.exe write2-test.exe       | sort
..\..\depends2.exe xmpparse.exe          | sort
..\..\depends2.exe xmpparser-test.exe    | sort
..\..\depends2.exe xmpsample.exe         | sort

cd ..\DebugDLL

echo -------DebugDLL---------------
..\..\depends2.exe addmoddel.exe         | sort
..\..\depends2.exe convert-test.exe      | sort
..\..\depends2.exe exifcomment.exe       | sort
..\..\depends2.exe exifdata-test.exe     | sort
..\..\depends2.exe exifprint.exe         | sort
..\..\depends2.exe exiv2.exe             | sort
..\..\depends2.exe exivsimple.dll        | sort
..\..\depends2.exe iotest.exe            | sort
..\..\depends2.exe iptceasy.exe          | sort
..\..\depends2.exe iptcprint.exe         | sort
..\..\depends2.exe iptctest.exe          | sort
..\..\depends2.exe largeiptc-test.exe    | sort
..\..\depends2.exe metacopy.exe          | sort
..\..\depends2.exe mmap-test.exe         | sort
..\..\depends2.exe prevtest.exe          | sort
..\..\depends2.exe stringto-test.exe     | sort
..\..\depends2.exe taglist.exe           | sort
..\..\depends2.exe tiff-test.exe         | sort
..\..\depends2.exe utiltest.exe          | sort
..\..\depends2.exe write-test.exe        | sort
..\..\depends2.exe write2-test.exe       | sort
..\..\depends2.exe xmpparse.exe          | sort
..\..\depends2.exe xmpparser-test.exe    | sort
..\..\depends2.exe xmpsample.exe         | sort

cd ..\Release

echo -------Release---------------
..\..\depends2.exe addmoddel.exe         | sort
..\..\depends2.exe convert-test.exe      | sort
..\..\depends2.exe exifcomment.exe       | sort
..\..\depends2.exe exifdata-test.exe     | sort
..\..\depends2.exe exifprint.exe         | sort
..\..\depends2.exe exiv2.exe             | sort
..\..\depends2.exe exivsimple.dll        | sort
..\..\depends2.exe iotest.exe            | sort
..\..\depends2.exe iptceasy.exe          | sort
..\..\depends2.exe iptcprint.exe         | sort
..\..\depends2.exe iptctest.exe          | sort
..\..\depends2.exe largeiptc-test.exe    | sort
..\..\depends2.exe metacopy.exe          | sort
..\..\depends2.exe mmap-test.exe         | sort
..\..\depends2.exe prevtest.exe          | sort
..\..\depends2.exe stringto-test.exe     | sort
..\..\depends2.exe taglist.exe           | sort
..\..\depends2.exe tiff-test.exe         | sort
..\..\depends2.exe utiltest.exe          | sort
..\..\depends2.exe write-test.exe        | sort
..\..\depends2.exe write2-test.exe       | sort
..\..\depends2.exe xmpparse.exe          | sort
..\..\depends2.exe xmpparser-test.exe    | sort
..\..\depends2.exe xmpsample.exe         | sort

cd ..\ReleaseDLL

echo -------ReleaseDLL---------------
..\..\depends2.exe addmoddel.exe         | sort
..\..\depends2.exe convert-test.exe      | sort
..\..\depends2.exe exifcomment.exe       | sort
..\..\depends2.exe exifdata-test.exe     | sort
..\..\depends2.exe exifprint.exe         | sort
..\..\depends2.exe exiv2.exe             | sort
..\..\depends2.exe exivsimple.dll        | sort
..\..\depends2.exe iotest.exe            | sort
..\..\depends2.exe iptceasy.exe          | sort
..\..\depends2.exe iptcprint.exe         | sort
..\..\depends2.exe iptctest.exe          | sort
..\..\depends2.exe largeiptc-test.exe    | sort
..\..\depends2.exe metacopy.exe          | sort
..\..\depends2.exe mmap-test.exe         | sort
..\..\depends2.exe prevtest.exe          | sort
..\..\depends2.exe stringto-test.exe     | sort
..\..\depends2.exe taglist.exe           | sort
..\..\depends2.exe tiff-test.exe         | sort
..\..\depends2.exe utiltest.exe          | sort
..\..\depends2.exe write-test.exe        | sort
..\..\depends2.exe write2-test.exe       | sort
..\..\depends2.exe xmpparse.exe          | sort
..\..\depends2.exe xmpparser-test.exe    | sort
..\..\depends2.exe xmpsample.exe         | sort

cd ..\..
