rmdir/s/q .\addmoddel\Debug
rmdir/s/q .\addmoddel\DebugDLL
rmdir/s/q .\addmoddel\Release
rmdir/s/q .\addmoddel\ReleaseDLL
rmdir/s/q .\organize\Debug
rmdir/s/q .\organize\DebugDLL
rmdir/s/q .\organize\Release
rmdir/s/q .\organize\ReleaseDLL
rmdir/s/q .\exifcomment\Debug
rmdir/s/q .\exifdata-test\Debug
rmdir/s/q .\exifprint\Debug
rmdir/s/q .\exiv2\Debug
rmdir/s/q .\exiv2lib\Debug
rmdir/s/q .\exivsimple\Debug
rmdir/s/q .\iotest\Debug
rmdir/s/q .\iptceasy\Debug
rmdir/s/q .\iptcprint\Debug
rmdir/s/q .\iptctest\Debug
rmdir/s/q .\largeiptc-test\Debug
rmdir/s/q .\metacopy\Debug
rmdir/s/q .\stringto-test\Debug
rmdir/s/q .\taglist\Debug
rmdir/s/q .\utiltest\Debug
rmdir/s/q .\write-test\Debug
rmdir/s/q .\write2-test\Debug
rmdir/s/q .\xmpparse\Debug
rmdir/s/q .\xmpparser-test\Debug
rmdir/s/q .\xmpsample\Debug
rmdir/s/q .\xmpsdk\Debug
rmdir/s/q .\exifcomment\DebugDLL
rmdir/s/q .\exifdata-test\DebugDLL
rmdir/s/q .\exifprint\DebugDLL
rmdir/s/q .\exiv2\DebugDLL
rmdir/s/q .\exiv2lib\DebugDLL
rmdir/s/q .\exivsimple\DebugDLL
rmdir/s/q .\iotest\DebugDLL
rmdir/s/q .\iptceasy\DebugDLL
rmdir/s/q .\iptcprint\DebugDLL
rmdir/s/q .\iptctest\DebugDLL
rmdir/s/q .\largeiptc-test\DebugDLL
rmdir/s/q .\metacopy\DebugDLL
rmdir/s/q .\stringto-test\DebugDLL
rmdir/s/q .\taglist\DebugDLL
rmdir/s/q .\utiltest\DebugDLL
rmdir/s/q .\write-test\DebugDLL
rmdir/s/q .\write2-test\DebugDLL
rmdir/s/q .\xmpparse\DebugDLL
rmdir/s/q .\xmpparser-test\DebugDLL
rmdir/s/q .\xmpsample\DebugDLL
rmdir/s/q .\xmpsdk\DebugDLL
rmdir/s/q .\exifcomment\Release
rmdir/s/q .\exifdata-test\Release
rmdir/s/q .\exifprint\Release
rmdir/s/q .\exiv2\Release
rmdir/s/q .\exiv2lib\Release
rmdir/s/q .\exivsimple\Release
rmdir/s/q .\iotest\Release
rmdir/s/q .\iptceasy\Release
rmdir/s/q .\iptcprint\Release
rmdir/s/q .\iptctest\Release
rmdir/s/q .\largeiptc-test\Release
rmdir/s/q .\metacopy\Release
rmdir/s/q .\stringto-test\Release
rmdir/s/q .\taglist\Release
rmdir/s/q .\utiltest\Release
rmdir/s/q .\write-test\Release
rmdir/s/q .\write2-test\Release
rmdir/s/q .\xmpparse\Release
rmdir/s/q .\xmpparser-test\Release
rmdir/s/q .\xmpsample\Release
rmdir/s/q .\xmpsdk\Release
rmdir/s/q .\exifcomment\ReleaseDLL
rmdir/s/q .\exifdata-test\ReleaseDLL
rmdir/s/q .\exifprint\ReleaseDLL
rmdir/s/q .\exiv2\ReleaseDLL
rmdir/s/q .\exiv2lib\ReleaseDLL
rmdir/s/q .\exivsimple\ReleaseDLL
rmdir/s/q .\iotest\ReleaseDLL
rmdir/s/q .\iptceasy\ReleaseDLL
rmdir/s/q .\iptcprint\ReleaseDLL
rmdir/s/q .\iptctest\ReleaseDLL
rmdir/s/q .\largeiptc-test\ReleaseDLL
rmdir/s/q .\metacopy\ReleaseDLL
rmdir/s/q .\stringto-test\ReleaseDLL
rmdir/s/q .\taglist\ReleaseDLL
rmdir/s/q .\utiltest\ReleaseDLL
rmdir/s/q .\write-test\ReleaseDLL
rmdir/s/q .\write2-test\ReleaseDLL
rmdir/s/q .\xmpparse\ReleaseDLL
rmdir/s/q .\xmpparser-test\ReleaseDLL
rmdir/s/q .\xmpsample\ReleaseDLL
rmdir/s/q .\xmpsdk\ReleaseDLL
rmdir/s/q .\prevtest\Release
rmdir/s/q .\prevtest\ReleaseDLL
rmdir/s/q .\prevtest\Debug
rmdir/s/q .\prevtest\DebugDLL
rmdir/s/q ..\..\expat-2.0.1\win32\bin\DebugDLL
rmdir/s/q ..\..\expat-2.0.1\win32\bin\ReleaseDLL

rmdir/s/q .\bin
rmdir/s/q include
del/s     *.user
del/s     *.old
attrib -h exiv2.suo
del/s     exiv2.suo
del/s     exiv2.ncb
